@@ -30,7 +30,6 @@
 #include <kdebug.h>
 #include <kglobal.h>
 #include <kshell.h>
-#include <kstringhandler.h>
 
 #include <stdio.h>
 #include <assert.h>
@@ -66,14 +65,14 @@ static QString cleanpath( const QString &_path, bool cleanDirSeparator, bool dec
      if (path.indexOf(encodedDot, 0, Qt::CaseInsensitive) != -1)
      {
         static const QString &encodedDOT = KGlobal::staticQString("%2E"); // Uppercase!
-        path.replace(encodedDot, ".");
-        path.replace(encodedDOT, ".");
+        path.replace(encodedDot, QString(QLatin1Char('.')));
+        path.replace(encodedDOT, QString(QLatin1Char('.')));
         len = path.length();
      }
   }
 
-  bool slash = (len && path[len-1] == QLatin1Char('/')) ||
-               (len > 1 && path[len-2] == QLatin1Char('/') && path[len-1] == QLatin1Char('.'));
+  const bool slash = (len && path[len-1] == QLatin1Char('/')) ||
+                     (len > 1 && path[len-2] == QLatin1Char('/') && path[len-1] == QLatin1Char('.'));
 
   // The following code cleans up directory path much like
   // QDir::cleanPath() except it can be made to ignore multiple
@@ -89,14 +88,14 @@ static QString cleanpath( const QString &_path, bool cleanDirSeparator, bool dec
   while ( pos && (pos = path.lastIndexOf(QLatin1Char('/'),--pos)) != -1 )
   {
     len = orig_pos - pos - 1;
-    if ( len == 2 && path[pos+1] == '.' && path[pos+2] == '.' )
+    if ( len == 2 && path[pos+1] == QLatin1Char('.') && path[pos+2] == QLatin1Char('.') )
       cdUp++;
     else
     {
       // Ignore any occurrences of '.'
       // This includes entries that simply do not make sense like /..../
       if ( (len || !cleanDirSeparator) &&
-           (len != 1 || path[pos+1] != '.' ) )
+           (len != 1 || path[pos+1] != QLatin1Char('.') ) )
       {
           if ( !cdUp )
               result.prepend(path.mid(pos, len+1));
@@ -114,9 +113,9 @@ static QString cleanpath( const QString &_path, bool cleanDirSeparator, bool dec
 #endif
 
   if ( result.isEmpty() )
-    result = '/';
+    result = QLatin1Char('/');
   else if ( slash && result[result.length()-1] != QLatin1Char('/') )
-       result.append(QChar('/'));
+       result.append(QLatin1Char('/'));
 
   return result;
 }
@@ -134,7 +133,7 @@ static QString removeSlashOrFilePrefix(const QString& str)
 {
   // FIXME this should maybe be replaced with some (faster?)/nicer logic
   const int len = str.length();
-  if (str[0]=='f') {
+  if (str[0]==QLatin1Char('f')) {
     if ( len > 10 && str.startsWith( QLatin1String( "file:///" ) )
          && IS_DRIVE_OR_DOUBLESLASH(str[8].isLetter(), str[8], str[9], QLatin1Char(':'), QLatin1Char('/')) )
       return QUrl::fromPercentEncoding( str.toLatin1() ).mid(8);
@@ -192,6 +191,13 @@ KUrl::List::List(const QList<KUrl> &list)
 {
 }
 
+KUrl::List::List(const QList<QUrl> &list)
+{
+    foreach(const QUrl& url, list) {
+        append(KUrl(url));
+    }
+}
+
 KUrl::List::List(const QStringList &list)
 {
   for (QStringList::ConstIterator it = list.begin();
@@ -204,14 +210,17 @@ KUrl::List::List(const QStringList &list)
 
 QStringList KUrl::List::toStringList() const
 {
-   QStringList lst;
-   for( KUrl::List::ConstIterator it = begin();
-        it != end();
-        ++it)
-   {
-      lst.append( (*it).url() );
-   }
-   return lst;
+    return toStringList(KUrl::LeaveTrailingSlash);
+}
+
+QStringList KUrl::List::toStringList(KUrl::AdjustPathOption trailing) const
+{
+    QStringList lst;
+    for(KUrl::List::ConstIterator it = constBegin();
+         it != constEnd(); ++it) {
+        lst.append(it->url(trailing));
+    }
+    return lst;
 }
 
 static QByteArray uriListData(const KUrl::List& urls)
@@ -240,7 +249,7 @@ void KUrl::List::populateMimeData( QMimeData* mimeData,
                                    const KUrl::MetaDataMap& metaData,
                                    MimeDataFlags flags ) const
 {
-    mimeData->setData("text/uri-list", uriListData(*this));
+    mimeData->setData(QString::fromLatin1("text/uri-list"), uriListData(*this));
 
     if ( ( flags & KUrl::NoTextExport ) == 0 )
     {
@@ -249,16 +258,16 @@ void KUrl::List::populateMimeData( QMimeData* mimeData,
         const KUrl::List::ConstIterator uEnd = constEnd();
         for ( ; uit != uEnd ; ++uit ) {
             QString prettyURL = (*uit).prettyUrl();
-            if ( (*uit).protocol() == "mailto" ) {
+            if ( (*uit).protocol() == QLatin1String("mailto") ) {
                 prettyURL = (*uit).path(); // remove mailto: when pasting into konsole
             }
             prettyURLsList.append( prettyURL );
         }
 
-        QByteArray plainTextData = prettyURLsList.join( "\n" ).toLocal8Bit();
+        QByteArray plainTextData = prettyURLsList.join(QString(QLatin1Char('\n'))).toLocal8Bit();
         if( count() > 1 ) // terminate last line, unless it's the only line
             plainTextData.append( "\n" );
-        mimeData->setData( "text/plain", plainTextData );
+        mimeData->setData( QString::fromLatin1("text/plain"), plainTextData );
     }
 
     if ( !metaData.isEmpty() )
@@ -271,7 +280,7 @@ void KUrl::List::populateMimeData( QMimeData* mimeData,
             metaDataData += it.value().toUtf8();
             metaDataData += "$@@$";
         }
-        mimeData->setData( "application/x-kio-metadata", metaDataData );
+        mimeData->setData( QString::fromLatin1("application/x-kio-metadata"), metaDataData );
     }
 }
 
@@ -284,18 +293,18 @@ void KUrl::List::populateMimeData(const KUrl::List& mostLocalUrls,
     // Export the most local urls as text/uri-list and plain text.
     mostLocalUrls.populateMimeData(mimeData, metaData, flags);
 
-    mimeData->setData(s_kdeUriListMime, uriListData(*this));
+    mimeData->setData(QString::fromLatin1(s_kdeUriListMime), uriListData(*this));
 }
 
 bool KUrl::List::canDecode( const QMimeData *mimeData )
 {
-    return mimeData->hasFormat("text/uri-list") ||
-        mimeData->hasFormat(s_kdeUriListMime);
+    return mimeData->hasFormat(QString::fromLatin1("text/uri-list")) ||
+        mimeData->hasFormat(QString::fromLatin1(s_kdeUriListMime));
 }
 
 QStringList KUrl::List::mimeDataTypes()
 {
-    return QStringList() << s_kdeUriListMime << "text/uri-list";
+    return QStringList() << QString::fromLatin1(s_kdeUriListMime) << QString::fromLatin1("text/uri-list");
 }
 
 
@@ -310,12 +319,12 @@ KUrl::List KUrl::List::fromMimeData(const QMimeData *mimeData,
     if (decodeOptions == PreferLocalUrls) {
         qSwap(firstMimeType, secondMimeType);
     }
-    QByteArray payload = mimeData->data(firstMimeType);
+    QByteArray payload = mimeData->data(QString::fromLatin1(firstMimeType));
     if (payload.isEmpty())
-        payload = mimeData->data(secondMimeType);
+        payload = mimeData->data(QString::fromLatin1(secondMimeType));
     if ( !payload.isEmpty() ) {
         int c = 0;
-        const char* d = payload.data();
+        const char* d = payload.constData();
         while ( c < payload.size() && d[c] ) {
             int f = c;
             // Find line end
@@ -333,13 +342,13 @@ KUrl::List KUrl::List::fromMimeData(const QMimeData *mimeData,
     }
     if ( metaData )
     {
-        const QByteArray metaDataPayload = mimeData->data( "application/x-kio-metadata" );
+        const QByteArray metaDataPayload = mimeData->data(QLatin1String("application/x-kio-metadata"));
         if ( !metaDataPayload.isEmpty() )
         {
             QString str = QString::fromUtf8( metaDataPayload );
-            Q_ASSERT( str.endsWith( "$@@$" ) );
+            Q_ASSERT(str.endsWith(QLatin1String("$@@$")));
             str.truncate( str.length() - 4 );
-            const QStringList lst = str.split( "$@@$" );
+            const QStringList lst = str.split(QLatin1String("$@@$"));
             QStringList::ConstIterator it = lst.begin();
             bool readingKey = true; // true, then false, then true, etc.
             QString key;
@@ -367,6 +376,15 @@ KUrl::List::operator QVariant() const
   return qVariantFromValue(*this);
 }
 
+KUrl::List::operator QList<QUrl>() const
+{
+    QList<QUrl> list;
+    foreach(const KUrl& url, *this) {
+        list << url;
+    }
+    return list;
+}
+
 ///
 
 KUrl::KUrl()
@@ -395,7 +413,7 @@ KUrl::KUrl( const QString &str )
     if ( !pathToSet.isEmpty() ) {
       // we have a prefix indicating this is a local URL
       // remember the possible query using _setEncodedUrl(), then set up the correct path without query protocol part
-      int index = pathToSet.lastIndexOf('?');
+      int index = pathToSet.lastIndexOf(QLatin1Char('?'));
       if (index == -1)
         setPath( pathToSet );
       else {
@@ -419,15 +437,15 @@ KUrl::KUrl( const char * str )
 #ifdef Q_WS_WIN
   // true if @a c is letter
   #define IS_LETTER(c) \
-    ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z'))
+    ((c >= QLatin1Char('A') && c <= QLatin1Char('Z')) || (c >= QLatin1Char('a') && c <= QLatin1Char('z')))
 
   // like IS_DRIVE_OR_DOUBLESLASH, but slash is prepended
   #define IS_SLASH_AND_DRIVE_OR_DOUBLESLASH_0 \
-    ( str[0] == '/' && IS_DRIVE_OR_DOUBLESLASH(IS_LETTER(str[1]), str[1], str[2], ':', '/') )
+    ( QLatin1Char(str[0]) == QLatin1Char('/') && IS_DRIVE_OR_DOUBLESLASH(IS_LETTER(QLatin1Char(str[1])), QLatin1Char(str[1]), QLatin1Char(str[2]), QLatin1Char(':'), QLatin1Char('/')) )
 
   // like IS_DRIVE_OR_DOUBLESLASH, with characters == str[0] and str[1]
   #define IS_DRIVE_OR_DOUBLESLASH_0 \
-    ( IS_DRIVE_OR_DOUBLESLASH(IS_LETTER(str[0]), str[0], str[1], ':', '/') )
+    ( IS_DRIVE_OR_DOUBLESLASH(IS_LETTER(QLatin1Char(str[0])), QLatin1Char(str[0]), QLatin1Char(str[1]), QLatin1Char(':'), QLatin1Char('/')) )
 
 #if defined(DEBUG_KURL)
   kDebug(kurlDebugArea()) << "KUrl::KUrl " << " " << str;
@@ -506,13 +524,13 @@ KUrl::KUrl( const KUrl& _u, const QString& _rel_url )
   // WORKAROUND THE RFC 1606 LOOPHOLE THAT ALLOWS
   // http:/index.html AS A VALID SYNTAX FOR RELATIVE
   // URLS. ( RFC 2396 section 5.2 item # 3 )
-  int len = _u.scheme().length();
+  const int len = _u.scheme().length();
   if ( !_u.host().isEmpty() && !rUrl.isEmpty() &&
        rUrl.indexOf( _u.scheme(), 0, Qt::CaseInsensitive ) == 0 &&
-       rUrl[len] == ':' && (rUrl[len+1] != QLatin1Char('/') ||
-       (rUrl[len+1] == '/' && rUrl[len+2] != QLatin1Char('/'))) )
+       rUrl[len] == QLatin1Char(':') && (rUrl[len+1] != QLatin1Char('/') ||
+       (rUrl[len+1] == QLatin1Char('/') && rUrl[len+2] != QLatin1Char('/'))) )
   {
-    rUrl.remove( 0, rUrl.indexOf( ':' ) + 1 );
+    rUrl.remove( 0, rUrl.indexOf( QLatin1Char(':') ) + 1 );
   }
 
 
@@ -520,7 +538,7 @@ KUrl::KUrl( const KUrl& _u, const QString& _rel_url )
   {
     *this = _u;
   }
-  else if ( rUrl[0] == '#' )
+  else if ( rUrl[0] == QLatin1Char('#') )
   {
     *this = _u;
     QByteArray strRef_encoded = rUrl.mid(1).toLatin1();
@@ -546,9 +564,9 @@ KUrl::KUrl( const KUrl& _u, const QString& _rel_url )
         }
         strPath.clear();
     }
-    else if ( rUrl[0] != '?' )
+    else if ( rUrl[0] != QLatin1Char('?') )
     {
-       int pos = strPath.lastIndexOf( QLatin1Char('/') );
+       const int pos = strPath.lastIndexOf( QLatin1Char('/') );
        if (pos >= 0)
           strPath.truncate(pos);
        strPath += QLatin1Char('/');
@@ -560,14 +578,14 @@ KUrl::KUrl( const KUrl& _u, const QString& _rel_url )
     }
     setPath( strPath );
     //kDebug(kurlDebugArea()) << "url()=" << url() << " rUrl=" << rUrl;
-    KUrl tmp( url() + rUrl);
+    const KUrl tmp( url() + rUrl);
     //kDebug(kurlDebugArea()) << "assigning tmp=" << tmp.url();
     *this = tmp;
     cleanPath(KeepDirSeparators);
   }
   else
   {
-    KUrl tmp( rUrl );
+    const KUrl tmp( rUrl );
     //kDebug(kurlDebugArea()) << "not relative; assigning tmp=" << tmp.url();
     *this = tmp;
     // Preserve userinfo if applicable.
@@ -602,10 +620,12 @@ KUrl::operator QVariant() const
   return qVariantFromValue(*this);
 }
 
+#ifndef KDE_NO_DEPRECATED
 bool KUrl::cmp( const KUrl &u, bool ignore_trailing ) const
 {
   return equals( u, ignore_trailing ? CompareWithoutTrailingSlash : EqualsOptions(0) );
 }
+#endif
 
 bool KUrl::equals( const KUrl &_u, const EqualsOptions& options ) const
 {
@@ -802,18 +822,18 @@ QString KUrl::encodedPathAndQuery( AdjustPathOption trailing , const EncodedPath
         encodedPath = trailingSlash(trailing, QUrl::toLocalFile());
         encodedPath = QString::fromLatin1(QUrl::toPercentEncoding(encodedPath, "!$&'()*+,;=:@/"));
     } else {
-        encodedPath = trailingSlash(trailing, QUrl::encodedPath());
+        encodedPath = trailingSlash(trailing, QString::fromLatin1(QUrl::encodedPath()));
     }
 #else
-    encodedPath = trailingSlash(trailing, QUrl::encodedPath());
+    encodedPath = trailingSlash(trailing, QString::fromLatin1(QUrl::encodedPath()));
 #endif
 
     if ((options & AvoidEmptyPath) && encodedPath.isEmpty()) {
-        encodedPath.append('/');
+        encodedPath.append(QLatin1Char('/'));
     }
 
     if (hasQuery()) {
-        return encodedPath + '?' + encodedQuery();
+        return encodedPath + QLatin1Char('?') + QString::fromLatin1(encodedQuery());
     } else {
         return encodedPath;
     }
@@ -836,7 +856,7 @@ void KUrl::setEncodedPath( const QString& _txt, int encoding_hint )
 
 void KUrl::setEncodedPathAndQuery( const QString& _txt )
 {
-  int pos = _txt.indexOf( '?' );
+  const int pos = _txt.indexOf(QLatin1Char('?'));
   if ( pos == -1 )
   {
     setPath( QUrl::fromPercentEncoding( _txt.toLatin1() ) );
@@ -852,7 +872,9 @@ void KUrl::setEncodedPathAndQuery( const QString& _txt )
 QString KUrl::path( AdjustPathOption trailing ) const
 {
 #ifdef Q_WS_WIN
+#ifdef DEBUG_KURL
   kWarning() << (isLocalFile() ? "converted to local file - the related call should be converted to toLocalFile()" : "") << QUrl::path();
+#endif
   return trailingSlash( trailing, isLocalFile() ? QUrl::toLocalFile() : QUrl::path() );
 #else
   return trailingSlash( trailing, QUrl::path() );
@@ -866,6 +888,14 @@ QString KUrl::toLocalFile( AdjustPathOption trailing ) const
         urlWithoutHost.setHost(QString());
         return trailingSlash(trailing, urlWithoutHost.toLocalFile());
     }
+#ifdef __GNUC__
+#warning FIXME: Remove #ifdef below once upstream bug, QTBUG-20322, is fixed. Also see BR# 194746.
+#endif
+#ifndef Q_WS_WIN
+    if (isLocalFile()) {
+        return trailingSlash(trailing, QUrl::path());
+    }
+#endif
     return trailingSlash(trailing, QUrl::toLocalFile());
 }
 
@@ -902,26 +932,26 @@ void KUrl::setFileEncoding(const QString &encoding)
 
   QString q = query();
 
-  if (!q.isEmpty() && (q[0] == '?'))
+  if (!q.isEmpty() && q[0] == QLatin1Char('?'))
      q = q.mid(1);
 
-  QStringList args = q.split('&', QString::SkipEmptyParts);
+  QStringList args = q.split(QLatin1Char('&'), QString::SkipEmptyParts);
   for(QStringList::Iterator it = args.begin();
       it != args.end();)
   {
       QString s = QUrl::fromPercentEncoding( (*it).toLatin1() );
-      if (s.startsWith("charset="))
+      if (s.startsWith(QLatin1String("charset=")))
          it = args.erase(it);
       else
          ++it;
   }
   if (!encoding.isEmpty())
-      args.append("charset=" + QUrl::toPercentEncoding(encoding));
+      args.append(QLatin1String("charset=") + QString::fromLatin1(QUrl::toPercentEncoding(encoding)));
 
   if (args.isEmpty())
      _setQuery(QString());
   else
-     _setQuery(args.join("&"));
+     _setQuery(args.join(QString(QLatin1Char('&'))));
 }
 
 QString KUrl::fileEncoding() const
@@ -934,16 +964,16 @@ QString KUrl::fileEncoding() const
   if (q.isEmpty())
      return QString();
 
-  if (q[0] == '?')
+  if (q[0] == QLatin1Char('?'))
      q = q.mid(1);
 
-  const QStringList args = q.split('&', QString::SkipEmptyParts);
+  const QStringList args = q.split(QLatin1Char('&'), QString::SkipEmptyParts);
   for(QStringList::ConstIterator it = args.begin();
       it != args.end();
       ++it)
   {
       QString s = QUrl::fromPercentEncoding((*it).toLatin1());
-      if (s.startsWith("charset="))
+      if (s.startsWith(QLatin1String("charset=")))
          return s.mid(8);
   }
   return QString();
@@ -953,44 +983,45 @@ inline static bool hasSubUrl( const QUrl& url )
 {
   // The isValid call triggers QUrlPrivate::validate which needs the full encoded url,
   // all this takes too much time for isLocalFile()
-  if ( url.scheme().isEmpty() /*|| !isValid()*/ )
+  const QString scheme = url.scheme();
+  if ( scheme.isEmpty() /*|| !isValid()*/ )
     return false;
-  const QByteArray ref( url.fragment().toLatin1() );
+  const QString ref( url.fragment() );
   if (ref.isEmpty())
      return false;
-  switch ( ref.data()[0] ) {
+  switch ( ref.at(0).unicode() ) {
   case 'g':
-    if ( ref.startsWith("gzip:") )
+    if ( ref.startsWith(QLatin1String("gzip:")) )
       return true;
     break;
   case 'b':
-    if ( ref.startsWith("bzip:") || ref.startsWith("bzip2:") )
+    if ( ref.startsWith(QLatin1String("bzip:")) || ref.startsWith(QLatin1String("bzip2:")) )
       return true;
     break;
   case 'l':
-    if ( ref.startsWith("lzma:") )
+    if ( ref.startsWith(QLatin1String("lzma:")) )
       return true;
     break;
   case 'x':
-    if ( ref.startsWith("xz:") )
+    if ( ref.startsWith(QLatin1String("xz:")) )
       return true;
     break;
   case 't':
-    if ( ref.startsWith("tar:") )
+    if ( ref.startsWith(QLatin1String("tar:")) )
       return true;
     break;
   case 'a':
-    if ( ref.startsWith("ar:") )
+    if ( ref.startsWith(QLatin1String("ar:")) )
       return true;
     break;
   case 'z':
-    if ( ref.startsWith("zip:") )
+    if ( ref.startsWith(QLatin1String("zip:")) )
       return true;
     break;
   default:
     break;
   }
-  if ( url.scheme() == "error" ) // anything that starts with error: has suburls
+  if ( scheme == QLatin1String("error") ) // anything that starts with error: has suburls
      return true;
   return false;
 }
@@ -1002,10 +1033,10 @@ bool KUrl::hasSubUrl() const
 
 QString KUrl::url( AdjustPathOption trailing ) const
 {
-    if (QString::compare(scheme(), QLatin1String("mailto"), Qt::CaseInsensitive) == 0) {
-        // mailto urls should be prettified, see the url183433 testcase.
-        return prettyUrl(trailing);
-    }
+  if (QString::compare(scheme(), QLatin1String("mailto"), Qt::CaseInsensitive) == 0) {
+      // mailto urls should be prettified, see the url183433 testcase.
+      return prettyUrl(trailing);
+  }
   if ( trailing == AddTrailingSlash && !path().endsWith( QLatin1Char('/') ) ) {
       // -1 and 0 are provided by QUrl, but not +1, so that one is a bit tricky.
       // To avoid reimplementing toEncoded() all over again, I just use another QUrl
@@ -1014,8 +1045,16 @@ QString KUrl::url( AdjustPathOption trailing ) const
       newUrl.setPath( path() + QLatin1Char('/') );
       return QString::fromLatin1(newUrl.toEncoded());
   }
-  else if ( trailing == RemoveTrailingSlash && path() == "/" ) {
-      return QLatin1String(toEncoded(None));
+  else if ( trailing == RemoveTrailingSlash) {
+      const QString cleanedPath = trailingSlash(trailing, path());
+      if (cleanedPath == QLatin1String("/")) {
+          if (path() != QLatin1String("/")) {
+              QUrl fixedUrl = *this;
+              fixedUrl.setPath(cleanedPath);
+              return QLatin1String(fixedUrl.toEncoded(None));
+          }
+          return QLatin1String(toEncoded(None));
+      }
   }
   return QString::fromLatin1(toEncoded(trailing == RemoveTrailingSlash ? StripTrailingSlash : None));
 }
@@ -1023,13 +1062,14 @@ QString KUrl::url( AdjustPathOption trailing ) const
 static QString toPrettyPercentEncoding(const QString &input, bool forFragment)
 {
   QString result;
+  result.reserve(input.length());
   for (int i = 0; i < input.length(); ++i) {
-    QChar c = input.at(i);
+    const QChar c = input.at(i);
     register ushort u = c.unicode();
     if (u < 0x20
         || (!forFragment && u == '?') // don't escape '?' in fragments, not needed and wrong (#173101)
         || u == '#' || u == '%'
-        || (u == ' ' && (i+1 == input.length() || input.at(i+1) == ' '))) {
+        || (u == ' ' && (i+1 == input.length() || input.at(i+1).unicode() == ' '))) {
       static const char hexdigits[] = "0123456789ABCDEF";
       result += QLatin1Char('%');
       result += QLatin1Char(hexdigits[(u & 0xf0) >> 4]);
@@ -1059,18 +1099,18 @@ QString KUrl::prettyUrl( AdjustPathOption trailing ) const
     if(!authority().isEmpty() || result == QLatin1String("file"))
         result += QLatin1String("://");
     else
-        result += QLatin1String(":");
+        result += QLatin1Char(':');
   }
 
   QString tmp = userName();
   if (!tmp.isEmpty()) {
-    result += QUrl::toPercentEncoding(tmp);
+    result += QString::fromLatin1(QUrl::toPercentEncoding(tmp));
     result += QLatin1Char('@');
   }
 
   // Check if host is an ipv6 address
   tmp = host();
-  if (tmp.contains(':'))
+  if (tmp.contains(QLatin1Char(':')))
     result += QLatin1Char('[') + tmp + QLatin1Char(']');
   else
     result += tmp;
@@ -1081,6 +1121,10 @@ QString KUrl::prettyUrl( AdjustPathOption trailing ) const
   }
 
   tmp = path();
+#ifdef Q_WS_WIN
+  if (isLocalFile())
+    tmp.prepend(QLatin1Char('/')); // KUrl::path() returns toLocalFile() on windows so we need to add the / back to create a proper url
+#endif
   result += toPrettyPercentEncoding(tmp, false);
 
   // adjust the trailing slash, if necessary
@@ -1091,7 +1135,7 @@ QString KUrl::prettyUrl( AdjustPathOption trailing ) const
 
   if (hasQuery()) {
     result += QLatin1Char('?');
-    result += encodedQuery();
+    result += QString::fromLatin1(encodedQuery());
   }
 
   if (hasFragment()) {
@@ -1164,7 +1208,7 @@ QString KUrl::toMimeDataString() const // don't fold this into populateMimeData,
 
 KUrl KUrl::fromMimeDataByteArray( const QByteArray& str )
 {
-  if ( str.startsWith( "file:" ) )
+  if ( str.startsWith( "file:" ) ) // krazy:exclude=strings
     return KUrl( str /*, QTextCodec::codecForLocale()->mibEnum()*/ );
 
   return KUrl( str /*, 106*/ ); // 106 is mib enum for utf8 codec;
@@ -1341,7 +1385,7 @@ QString KUrl::directory( const DirectoryOptions& options ) const
   if ( result.isEmpty() || result == QLatin1String ( "/" ) )
     return result;
 
-  int i = result.lastIndexOf( '/' );
+  int i = result.lastIndexOf( QLatin1Char('/') );
   // If ( i == -1 ) => the first character is not a '/'
   // So it's some URL like file:blah.tgz, with no path
   if ( i == -1 )
@@ -1349,7 +1393,7 @@ QString KUrl::directory( const DirectoryOptions& options ) const
 
   if ( i == 0 )
   {
-    return QLatin1String( "/" );
+    return QString(QLatin1Char('/'));
   }
 
 #ifdef Q_WS_WIN
@@ -1400,7 +1444,7 @@ bool KUrl::cd( const QString& _dir )
   }
 
   // Users home directory on the local disk ?
-  if ( ( _dir[0] == '~' ) && ( scheme() == QLatin1String ( "file" ) ))
+  if (_dir[0] == QLatin1Char('~') && scheme() == QLatin1String ("file"))
   {
     //m_strPath_encoded.clear();
     QString strPath = QDir::homePath();
@@ -1443,9 +1487,7 @@ KUrl KUrl::upUrl( ) const
   if (!hasSubUrl())
   {
      KUrl u(*this);
-
-     u.cd("../");
-
+     u.cd(QLatin1String("../"));
      return u;
   }
 
@@ -1457,7 +1499,7 @@ KUrl KUrl::upUrl( ) const
   {
      KUrl &u = lst.last();
      const QString old = u.path();
-     u.cd("../");
+     u.cd(QLatin1String("../"));
      if (u.path() != old)
          break; // Finished.
      if (lst.count() == 1)
@@ -1471,11 +1513,11 @@ QString KUrl::htmlRef() const
 {
   if ( !hasSubUrl() )
   {
-      return QUrl::fromPercentEncoding( ref().toLatin1() );
+      return fragment();
   }
 
   const List lst = split( *this );
-  return QUrl::fromPercentEncoding( (*lst.begin()).ref().toLatin1() );
+  return (*lst.begin()).fragment();
 }
 
 QString KUrl::encodedHtmlRef() const
@@ -1525,8 +1567,8 @@ void KUrl::setDirectory( const QString &dir)
 
 void KUrl::setQuery( const QString &_txt )
 {
-  if (!_txt.isEmpty() && _txt[0] == '?')
-    _setQuery( _txt.length() > 1 ? _txt.mid(1) : "" /*empty, not null*/ );
+  if (!_txt.isEmpty() && _txt[0] == QLatin1Char('?'))
+    _setQuery( _txt.length() > 1 ? _txt.mid(1) : QString::fromLatin1("") /*empty, not null*/ );
   else
     _setQuery( _txt );
 }
@@ -1536,7 +1578,7 @@ void KUrl::_setQuery( const QString& query )
     if ( query.isNull() ) {
         setEncodedQuery( QByteArray() );
     } else if ( query.isEmpty() ) {
-        setEncodedQuery( "" );
+        setEncodedQuery("");
     } else {
         setEncodedQuery( query.toLatin1() ); // already percent-escaped, so toLatin1 is ok
     }
@@ -1547,16 +1589,17 @@ QString KUrl::query() const
   if (!hasQuery()) {
     return QString();
   }
-  return QString( QChar( '?' ) ) + QString::fromAscii( encodedQuery() );
+  return QString(QLatin1Char('?')) + QString::fromLatin1(encodedQuery());
 }
 
 void KUrl::_setEncodedUrl(const QByteArray& url)
 {
   setEncodedUrl(url, QUrl::TolerantMode);
   if (!isValid()) // see unit tests referring to N183630/task 183874
-    setUrl(url, QUrl::TolerantMode);
+    setUrl(QString::fromUtf8(url), QUrl::TolerantMode);
 }
 
+#ifndef KDE_NO_DEPRECATED
 bool urlcmp( const QString& _url1, const QString& _url2 )
 {
   return QUrl( _url1, QUrl::TolerantMode ) == QUrl( _url2, QUrl::TolerantMode );
@@ -1578,7 +1621,9 @@ bool urlcmp( const QString& _url1, const QString& _url2 )
   return ( list1 == list2 );
 #endif
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 bool urlcmp( const QString& _url1, const QString& _url2, const KUrl::EqualsOptions& _options )
 {
     // Both empty ?
@@ -1619,14 +1664,16 @@ bool urlcmp( const QString& _url1, const QString& _url2, const KUrl::EqualsOptio
   return true;
 #endif
 }
+#endif
 
 // static
+#ifndef KDE_NO_DEPRECATED
 KUrl KUrl::fromPathOrUrl( const QString& text )
 {
     KUrl url;
     if ( !text.isEmpty() )
     {
-        if (!QDir::isRelativePath(text) || text[0] == '~')
+        if (!QDir::isRelativePath(text) || text[0] == QLatin1Char('~'))
             url.setPath( text );
         else
             url = KUrl( text );
@@ -1634,11 +1681,12 @@ KUrl KUrl::fromPathOrUrl( const QString& text )
 
     return url;
 }
+#endif
 
 static QString _relativePath(const QString &base_dir, const QString &path, bool &isParent)
 {
    QString _base_dir(QDir::cleanPath(base_dir));
-   QString _path(QDir::cleanPath(path.isEmpty() || (path[0] != QLatin1Char('/')) ? _base_dir+'/'+path : path));
+   QString _path(QDir::cleanPath(path.isEmpty() || (path[0] != QLatin1Char('/')) ? _base_dir+QLatin1Char('/')+path : path));
 
    if (_base_dir.isEmpty())
       return _path;
@@ -1657,11 +1705,11 @@ static QString _relativePath(const QString &base_dir, const QString &path, bool
    QString result;
    // Need to go down out of the first path to the common branch.
    for(int i = level; i < list1.count(); i++)
-      result.append("../");
+      result.append(QLatin1String("../"));
 
    // Now up up from the common branch to the second path.
    for(int i = level; i < list2.count(); i++)
-      result.append(list2[i]).append("/");
+      result.append(list2[i]).append(QLatin1Char('/'));
 
    if ((level < list2.count()) && (path[path.length()-1] != QLatin1Char('/')))
       result.truncate(result.length()-1);
@@ -1676,7 +1724,7 @@ QString KUrl::relativePath(const QString &base_dir, const QString &path, bool *i
    bool parent = false;
    QString result = _relativePath(base_dir, path, parent);
    if (parent)
-      result.prepend("./");
+      result.prepend(QLatin1String("./"));
 
    if (isParent)
       *isParent = parent;
@@ -1708,12 +1756,12 @@ QString KUrl::relativeUrl(const KUrl &base_url, const KUrl &url)
 
    if ( url.hasRef() )
    {
-      relURL += '#';
+      relURL += QLatin1Char('#');
       relURL += url.ref();
    }
 
    if ( relURL.isEmpty() )
-      return "./";
+      return QLatin1String("./");
 
    return relURL;
 }
@@ -1754,15 +1802,16 @@ QMap< QString, QString > KUrl::queryItems( int options ) const {
 }
 #endif
 
-QMap< QString, QString > KUrl::queryItems( const QueryItemsOptions &options ) const {
-  const QString strQueryEncoded = encodedQuery();
+QMap< QString, QString > KUrl::queryItems( const QueryItemsOptions &options ) const
+{
+  const QString strQueryEncoded = QString::fromLatin1(encodedQuery());
   if ( strQueryEncoded.isEmpty() )
     return QMap<QString,QString>();
 
   QMap< QString, QString > result;
-  const QStringList items = strQueryEncoded.split( '&', QString::SkipEmptyParts );
+  const QStringList items = strQueryEncoded.split( QLatin1Char('&'), QString::SkipEmptyParts );
   for ( QStringList::const_iterator it = items.begin() ; it != items.end() ; ++it ) {
-    const int equal_pos = (*it).indexOf( '=' );
+    const int equal_pos = (*it).indexOf(QLatin1Char('='));
     if ( equal_pos > 0 ) { // = is not the first char...
       QString name = (*it).left( equal_pos );
       if ( options & CaseInsensitiveKeys )
@@ -1772,7 +1821,7 @@ QMap< QString, QString > KUrl::queryItems( const QueryItemsOptions &options ) co
         result.insert( name, QString::fromLatin1("") );
       else {
 	// ### why is decoding name not necessary?
-	value.replace( '+', ' ' ); // + in queries means space
+	value.replace( QLatin1Char('+'), QLatin1Char(' ') ); // + in queries means space
 	result.insert( name, QUrl::fromPercentEncoding( value.toLatin1() ) );
       }
     } else if ( equal_pos < 0 ) { // no =
@@ -1788,12 +1837,12 @@ QMap< QString, QString > KUrl::queryItems( const QueryItemsOptions &options ) co
 
 QString KUrl::queryItem( const QString& _item ) const
 {
-  const QString strQueryEncoded = encodedQuery();
-  const QString item = _item + '=';
+  const QString strQueryEncoded = QString::fromLatin1(encodedQuery());
+  const QString item = _item + QLatin1Char('=');
   if ( strQueryEncoded.length() <= 1 )
     return QString();
 
-  const QStringList items = strQueryEncoded.split( '&', QString::SkipEmptyParts );
+  const QStringList items = strQueryEncoded.split( QString(QLatin1Char('&')), QString::SkipEmptyParts );
   const int _len = item.length();
   for ( QStringList::ConstIterator it = items.begin(); it != items.end(); ++it )
   {
@@ -1802,7 +1851,7 @@ QString KUrl::queryItem( const QString& _item ) const
       if ( (*it).length() > _len )
       {
         QString str = (*it).mid( _len );
-        str.replace( '+', ' ' ); // + in queries means space.
+        str.replace( QLatin1Char('+'), QLatin1Char(' ') ); // + in queries means space.
         return QUrl::fromPercentEncoding( str.toLatin1() );
       }
       else // empty value
@@ -1815,12 +1864,12 @@ QString KUrl::queryItem( const QString& _item ) const
 
 void KUrl::addQueryItem( const QString& _item, const QString& _value )
 {
-  QString item = _item + '=';
-  QString value = QUrl::toPercentEncoding( _value );
+  QString item = _item + QLatin1Char('=');
+  QString value = QString::fromLatin1(QUrl::toPercentEncoding(_value));
 
-  QString strQueryEncoded = encodedQuery();
+  QString strQueryEncoded = QString::fromLatin1(encodedQuery());
   if (!strQueryEncoded.isEmpty())
-     strQueryEncoded += '&';
+     strQueryEncoded += QLatin1Char('&');
   strQueryEncoded += item + value;
   setEncodedQuery( strQueryEncoded.toLatin1() );
 }
@@ -1840,8 +1889,8 @@ bool KUrl::hasRef() const
 
 void KUrl::setRef( const QString& fragment )
 {
-  if ( fragment.isNull() )
-    setFragment( fragment ); // pass null, not empty
+  if ( fragment.isEmpty() ) // empty or null
+    setFragment( fragment );
   else
     setFragment( QUrl::fromPercentEncoding( fragment.toLatin1() ) );
 }
