@@ -29,9 +29,10 @@ private Q_SLOTS:
     void testLocateLocal();
     void testSaveLocation();
     void testAppData();
+    void testChangeSaveLocation();
     void testFindResource();
     void testFindAllResources();
-    void testFindAllResourcesNewDir();    
+    void testFindAllResourcesNewDir();
     void testFindDirs();
     void testFindResourceDir();
     void testFindExe();
@@ -41,6 +42,7 @@ private Q_SLOTS:
     void testAddResourceDir();
     void testSetXdgDataDirs();
     void testRestrictedResources();
+    void testSymlinkResolution();
     void testThreads();
 
 private:
