@@ -3,6 +3,7 @@
  *  This file is part of the KDE libraries
  *  Copyright (C) 2001 Peter Kelly (pmk@post.com)
  *  Copyright (C) 2003 Apple Computer, Inc.
+ *  Copyright (C) 2006, 2009, 2010 Maksim Orlovich (maksim@kde.org)
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -22,6 +23,7 @@
 #include "kjs_events.h"
 #include "kjs_events.lut.h"
 
+#include "kjs_data.h"
 #include "kjs_window.h"
 #include "kjs_views.h"
 #include "kjs_proxy.h"
@@ -48,14 +50,14 @@ JSEventListener::JSEventListener(JSObject *_listener, JSObject *_compareListener
 {
     //fprintf(stderr,"JSEventListener::JSEventListener this=%p listener=%p\n",this,listener.imp());
   if (compareListenerImp) {
-    static_cast<Window*>(win.get())->jsEventListeners.insert(QPair<void*, bool>(compareListenerImp, html), this);
+    static_cast<Window*>(win.get())->jsEventListeners.insert(QPair<void*, bool>(compareListenerImp.get(), html), this);
   }
 }
 
 JSEventListener::~JSEventListener()
 {
   if (compareListenerImp) {
-    static_cast<Window*>(win.get())->jsEventListeners.remove(QPair<void*, bool>(compareListenerImp, html));
+    static_cast<Window*>(win.get())->jsEventListeners.remove(QPair<void*, bool>(compareListenerImp.get(), html));
   }
   //fprintf(stderr,"JSEventListener::~JSEventListener this=%p listener=%p\n",this,listener.imp());
 }
@@ -90,9 +92,9 @@ void JSEventListener::handleEvent(DOM::Event &evt)
     } else {
       thisObj = compareListenerImp;
     }
+
     if ( !thisObj ) {
-      // Window events (window.onload/window.onresize etc.) must have 'this' set to the window.
-      // DocumentImpl::defaultEventHandler sets currentTarget to 0 to mean 'window'.
+      // ### can this still happen? eventTarget should be window on Window events now.
       thisObj = win;
     }
 
@@ -223,7 +225,7 @@ void JSLazyEventListener::parseCode() const
     code.clear();
 
     if (listener) {
-      static_cast<Window*>(win.get())->jsEventListeners.insert(QPair<void*, bool>(compareListenerImp, true),
+      static_cast<Window*>(win.get())->jsEventListeners.insert(QPair<void*, bool>(compareListenerImp.get(), true),
                                               (KJS::JSEventListener *)(this));
     }
 
@@ -379,6 +381,8 @@ JSValue *KJS::getDOMEvent(ExecState *exec, DOM::EventImpl* ei)
       ret = new DOMUIEvent(exec, static_cast<DOM::UIEventImpl*>(ei));
     else if (ei->isMutationEvent())
       ret = new DOMMutationEvent(exec, static_cast<DOM::MutationEventImpl*>(ei));
+    else if (ei->isMessageEvent())
+      ret = new DOMMessageEvent(exec, static_cast<DOM::MessageEventImpl*>(ei));
     else
       ret = new DOMEvent(exec, ei);
 
@@ -714,7 +718,7 @@ const ClassInfo DOMKeyEventBase::info = { "KeyEventBase", &DOMUIEvent::info, &DO
   ctrlKey        DOMKeyEventBase::CtrlKey     DontDelete|ReadOnly
   altKey         DOMKeyEventBase::AltKey      DontDelete|ReadOnly
   shiftKey       DOMKeyEventBase::ShiftKey    DontDelete|ReadOnly
-  altKey         DOMKeyEventBase::AltKey      DontDelete|ReadOnly
+  metaKey        DOMKeyEventBase::MetaKey     DontDelete|ReadOnly
 @end
 */
 
@@ -1018,3 +1022,85 @@ JSValue *DOMMutationEventProtoFunc::callAsFunction(ExecState *exec, JSObject *th
   }
   return jsUndefined();
 }
+// -------------------------------------------------------------------------
+
+const ClassInfo DOMMessageEvent::info = { "MessageEvent", &DOMEvent::info, &DOMMessageEventTable, 0 };
+/*
+@begin DOMMessageEventTable 5
+  data     DOMMessageEvent::Data     DontDelete|ReadOnly
+  origin   DOMMessageEvent::Origin   DontDelete|ReadOnly
+  source   DOMMessageEvent::Source   DontDelete|ReadOnly
+  lastEventId  DOMMessageEvent::LastEventId   DontDelete|ReadOnly
+@end
+@begin DOMMessageEventProtoTable 1
+  initMessageEvent     DOMMessageEvent::InitMessageEvent     DontDelete|Function 7
+@end
+*/
+KJS_DEFINE_PROTOTYPE(DOMMessageEventProto)
+KJS_IMPLEMENT_PROTOFUNC(DOMMessageEventProtoFunc)
+KJS_IMPLEMENT_PROTOTYPE("DOMMessageEvent",DOMMessageEventProto,DOMMessageEventProtoFunc,DOMEventProto)
+IMPLEMENT_PSEUDO_CONSTRUCTOR(MessageEventPseudoCtor, "DOMMessageEvent", DOMMessageEventProto)
+
+DOMMessageEvent::DOMMessageEvent(ExecState *exec, DOM::MessageEventImpl* me) :
+  DOMEvent(DOMMessageEventProto::self(exec), me) {}
+
+bool DOMMessageEvent::getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot)
+{
+  return getStaticValueSlot<DOMMessageEvent, DOMEvent>(exec,&DOMMessageEventTable,this,propertyName,slot);
+}
+
+JSValue *DOMMessageEvent::getValueProperty(ExecState *exec, int token) const
+{
+  DOM::MessageEventImpl& event = *impl();
+  switch (token) {
+  case Data:
+    return getMessageEventData(exec, event.data().get());
+  case Origin:
+    return jsString(event.origin());
+  case LastEventId:
+    return jsString(event.lastEventId());
+  case Source: 
+    if (KHTMLPart* p = event.source())
+	return Window::retrieve(p);
+    else
+	return jsNull();
+  default:
+    kDebug(6070) << "WARNING: Unhandled token in DOMMessageEvent::getValueProperty : " << token;
+    return 0;
+  }
+}
+
+JSValue *DOMMessageEventProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const List &args)
+{
+    KJS_CHECK_THIS( KJS::DOMMessageEvent, thisObj );
+    DOM::MessageEventImpl& messageEvent = *static_cast<DOMMessageEvent *>(thisObj)->impl();
+    switch (id) {
+        case DOMMessageEvent::InitMessageEvent: {
+            JSObject* sourceObj = args[3]->getObject();
+
+            Window* sourceWin = 0;
+            if (sourceObj && sourceObj->inherits(&Window::info))
+                sourceWin = static_cast<Window *>(sourceObj);
+
+            KHTMLPart* part = 0;
+            if (sourceWin)
+                part = qobject_cast<KHTMLPart*>(sourceWin->part());
+
+            if (!part) {
+                setDOMException(exec, DOM::DOMException::TYPE_MISMATCH_ERR);
+                return jsUndefined();
+            }
+        
+            messageEvent.initMessageEvent(args[0]->toString(exec).domString(), // typeArg,
+                                          args[1]->toBoolean(exec), // canBubbleArg
+                                          args[2]->toBoolean(exec), // cancelableArg
+                                          encapsulateMessageEventData(
+                                               exec, exec->dynamicInterpreter(), args[3]), // dataArg
+                                          args[4]->toString(exec).domString(), // originArg
+                                          args[5]->toString(exec).domString(), // lastEventIdArg
+                                          part); // sourceArg
+            return jsUndefined();
+        }
+    }
+    return jsUndefined();
+}
